/**
 * @file noachiand/controller.h
 * @brief noachiand controller interface
 */

/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, version 3, as published by the
 * Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef NOACHIAND_CONTROLLER_H
#define NOACHIAND_CONTROLLER_H 1

#include <memory>
#include <string>
#include <vector>


namespace Noachiand
{
class Configuration;
class TrustedSessionFactory;
class ContainerFactory;

/**
 * A master controller for aggregating the various components of the Noachian
 * service.
 */
class Controller
{
public:
  Controller(Configuration*         configuration,
             TrustedSessionFactory* trusted_session_factory,
             ContainerFactory*      container_factory);
  ~Controller() ;

  /**
   * Starts an application.
   * @param[in]  application_path Path of application to invoke (may be a
   *                              relative path).
   * @param[in]  argv             Arguments to send to application on
   *                              invocation.
   * @param[out] id               Identifier of running program instance.
   *
   * @throws std::runtime_error on failre to start the application.
   */
  void
  start_application(std::string const&              application_path,
                    std::vector<std::string> const& argv,
                    uint32_t*                       id);

private:
  struct Impl;
  std::unique_ptr<Impl> impl_;
};

} // namespace Noachiand

#endif /* NOACHIAND_CONTROLLER_H */
