/**
 * @file noachiand/main.cpp
 * @brief noachiand program entry point.
 */

/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, version 3, as published by the
 * Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include <glib.h>
#include <glib-unix.h>
#include <memory>
#include "noachiand/configuration.h"
#include "noachiand/controller.h"
#include "noachiand/dbusservice.h"
#include <stdexcept>


/** C++ wrapper for GMainLoop */
typedef std::unique_ptr<GMainLoop, void(*)(GMainLoop*)>  main_loop_t;


/**
 * Signal handler for INT and TERM signals
 *
 * Shuts the daemon down gracefully.
 */
static gboolean
quit_cb(gpointer loop)
{
  if (loop)
    g_idle_add((GSourceFunc)g_main_loop_quit, loop);
  else
    exit(0);
  return FALSE;
}


int
main (int argc, char *argv[])
{
  Noachiand::Configuration configuration(argc, argv);
  Noachiand::Controller    controller(&configuration, NULL, NULL);
  Noachiand::DbusService   dbus_service(&configuration, &controller);

  main_loop_t main_loop(g_main_loop_new(NULL, FALSE), g_main_loop_unref);
  g_unix_signal_add(SIGTERM, quit_cb, main_loop.get());
  g_unix_signal_add(SIGINT,  quit_cb, main_loop.get());
  g_main_loop_run(main_loop.get());
  return 0;
}
