/**
 * @file test/mockcontainerfactory.h
 * @brief concrete mock  container factory
 */

/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, version 3, as published by the
 * Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef TEST_MOCKCONTAINERFACTORY_H
#define TEST_MOCKCONTAINERFACTORY_H 1

#include "noachiand/containerfactory.h"


namespace Test
{

/**
 * An factory that produces Mock Containers.
 *
 * The English language is a slippery thing:  this is a factory of
 * MockContainers, not a Mock ContainerFactory.
 *
 * The idea is this factory injects a mock implementation into a Container, and
 * any tests can examine operations on the mock object at their leisure.
 */
class MockContainerFactory
: public Noachiand::ContainerFactory
{
public:
  ~MockContainerFactory() override;

  void
  set_mock_container_impl(Noachiand::Container::ImplPtr const& impl);

  /** Creates a container. */
  Noachiand::Container::Ptr
  create_container() override;

private:
  Noachiand::Container::ImplPtr impl_;
};

} // namespace Test

#endif /* TEST_MOCKCONTAINERFACTORY_H */
